#include "rarccorruptioninfo.h"

RARCCorruptionInfo::RARCCorruptionInfo(std::vector<std::string>& args)
{
  for (uint32_t i = 0; i < args.size(); i++)
  {
    auto& arg = args[i];

    if (arg == "-s" || arg == "--step")
    {
      this->step = Util::to_int32(args[++i]);
    }
    else if (arg == "-h" || arg == "--shift")
    {
      this->value = this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::Shift;
    }
    else if (arg == "-w" || arg == "--swap")
    {
      this->value = this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::Swap;
    }
    else if (arg == "-a" || arg == "--add")
    {
      this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::Add;
    }
    else if (arg == "-t" || arg == "--set")
    {
      this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::Set;
    }
    else if (arg == "-r" || arg == "--random")
    {
      this->type = CorruptionType::Random;
    }
    else if (arg == "--rotate-left")
    {
      this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::RotateLeft;
    }
    else if (arg == "--rotate-right")
    {
      this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::RotateRight;
    }
    else if (arg == "--logical-and")
    {
      this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::LogicalAnd;
    }
    else if (arg == "--logical-or")
    {
      this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::LogicalOr;
    }
    else if (arg == "--logical-xor")
    {
      this->value = Util::to_int32(args[++i]);
      this->type = CorruptionType::LogicalXor;
    }
    else if (arg == "--logical-complement")
    {
      this->type = CorruptionType::LogicalComplement;
    }
    else if (arg == "-b" || arg == "--start")
    {
      this->start = Util::to_int32(args[++i]);
    }
    else if (arg == "-e" || arg == "--stop")
    {
      this->end = Util::to_int32(args[++i]);
    }
    else if (arg == "-o" || arg == "--out")
    {
      this->outfile = args[i + 1];
      i++;
    }
    else if (arg == "-f" || arg == "--files")
    {
      for (uint32_t file = i + 1; file < args.size() && args[file][0] != '-'; file++)
      {
        this->add_file(args[++i]);
      }
    }
    else if (arg == "--filelist")
    {
      std::ifstream ifs(args[i + 1]);

      if (ifs.is_open())
      {
        std::string raw((std::istreambuf_iterator<char>(ifs)), std::istreambuf_iterator<char>());
        std::vector<std::string> files;

        boost::split(files, raw, boost::is_any_of(" "), boost::token_compress_on);

        for (auto& file : files)
        {
          this->add_file(file);
        }
      }
      else
      {
        throw InvalidArgumentException("Could not open file " + args[i + 1]);
      }

      ifs.close();
      i++;
    }
    else
    {
      throw InvalidArgumentException(arg + " is not a recognized flag.");
    }
  }
}

RARCCorruptionInfo::~RARCCorruptionInfo() {}